
//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 MaterialAmbientColor = {0.3, 0.3, 0.3, 1.0};
float4 MaterialDiffuseColor = {0.7, 0.7, 0.7, 0.0};

#define LIGHT_DIR  0.0,10.0,0.0
float3 LightDir = normalize(float3(LIGHT_DIR));
float4 LightAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
float4 LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

float4x4 mWorld;
float4x4 mWorldView;
float4x4 mWorldViewProjection;

float weight[4];
static const float MAP_WIDTH  = 1024;
static const float MAP_HEIGHT = 1024;
float4 vBias = {2/MAP_WIDTH, 4/MAP_WIDTH, 6/MAP_WIDTH, 8/MAP_WIDTH};
float4 hBias = {2/MAP_HEIGHT,4/MAP_HEIGHT,6/MAP_HEIGHT,8/MAP_HEIGHT};

float fIntensity = 1.0;

float z_max ;
float z_min;

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture RenderTargetTexture;
sampler RenderTargetSampler = 
sampler_state
{
    Texture = <RenderTargetTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture MeshTexture;
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <MeshTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};


texture DepthRenderTarget;
sampler DepthSampler = 
sampler_state
{
    Texture = <DepthRenderTarget>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};






//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
float3 GetDiffuse(float3 normalW )
{
    return MaterialDiffuseColor * LightDiffuse * max(0,dot(normalW, LightDir))
           + MaterialAmbientColor * LightAmbient;
}


//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position : POSITION;
    float3 TextureUV : TEXCOORD0;
    float4 Diffuse : TEXCOORD1;
};
struct PS_OUTPUT
{
    float4 Color : Color0;
    float4 Depth : Color1;
};


//-----------------------------------------------------------------------------
// Name: SceneVertexShader
// Type: Vertex shader                                      
// Desc: In addition to standard transform and lighting, it calculates the blur
//       factor of the vertex and outputs this as a texture coord.
//-----------------------------------------------------------------------------
VS_OUTPUT SceneVertexShader( float4 vPos : POSITION, 
                             float3 vNormal : NORMAL,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    float3 vViewPosition;
    float3 vWorldNormal;
    float  fBlurFactor;
  
    // tranform vertex position into screen space
    Output.Position = mul(vPos, mWorldViewProjection);
    
    // Compute simple lighting equation
    vWorldNormal = mul(vNormal, (float3x3)mWorld);       
    Output.Diffuse = float4(GetDiffuse(vWorldNormal), 1.0);
    
    // Just copy the texture coordinate through
    Output.TextureUV.xy = vTexCoord0;
    
    // [xeNX`Wɖߍ
    Output.TextureUV.z = (mul(vPos, mWorldView).z - z_min)/(z_max-z_min);

    return Output;
}


//-----------------------------------------------------------------------------
// Name: ScenePixelShader
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
PS_OUTPUT ScenePixelShader( VS_OUTPUT In )
{
    PS_OUTPUT Output;

    Output.Color = tex2D(MeshTextureSampler, In.TextureUV.xy) * In.Diffuse;
    Output.Depth = In.TextureUV.z;

    return Output;
}


//-----------------------------------------------------------------------------
// Name: TechScene
// Type: Technique                                     
// Desc: Renders the scene's color to the render target and stores the
//       depth information as a blur factor in the alpha channel.
//-----------------------------------------------------------------------------
technique TechScene
{
    pass P0
    {        
        VertexShader = compile vs_1_1 SceneVertexShader();
        PixelShader  = compile ps_2_0 ScenePixelShader();
    }
}




//-----------------------------------------------------------------------------
// Name: RenderGaussX
//-----------------------------------------------------------------------------
float4 RenderGaussX( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
     float4 Color;
     
     Color  = weight[0] *  tex2D( MeshTextureSampler, OriginalUV );
     Color += weight[1]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( vBias.x, 0 ) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( vBias.x, 0 ) ));
     Color += weight[2]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( vBias.y, 0 ) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( vBias.y, 0 ) ));
     Color += weight[3]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( vBias.z, 0 ) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( vBias.z, 0 ) ));
     
     return Color;
}


technique TechGaussX
{
    pass P0
    {        
        PixelShader = compile ps_2_0 RenderGaussX();
    }
}
//-----------------------------------------------------------------------------
// Name: RenderGaussX
//-----------------------------------------------------------------------------
float4 RenderGaussY( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
     float4 Color;

     Color  = weight[0] *  tex2D( MeshTextureSampler, OriginalUV );
     Color += weight[1]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( 0, hBias.x) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( 0, hBias.x) ));
     Color += weight[2]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( 0, hBias.y) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( 0, hBias.y) ));
     Color += weight[3]
      * (tex2D( MeshTextureSampler, OriginalUV + float2( 0, hBias.z) )
      +  tex2D( MeshTextureSampler, OriginalUV - float2( 0, hBias.z) ));
     
     return Color;
}


technique TechGaussY
{
    pass P0
    {        
        PixelShader = compile ps_2_0 RenderGaussY();
    }
}



float4 FinalPS( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
	float4 color = tex2D(RenderTargetSampler, OriginalUV);
	float  depth      = tex2D(DepthSampler, OriginalUV).r;
	float  depth_blur = tex2D(MeshTextureSampler, OriginalUV).r;

	float diff = depth - depth_blur;
	diff = max(0, diff);

	return color * (1 - fIntensity * diff);
}


//-----------------------------------------------------------------------------
// Name: TechFishEye
// Type: Technique
//-----------------------------------------------------------------------------
technique TechFinal
{
    pass P0
    {        
        PixelShader = compile ps_2_0 FinalPS();
        
        ZEnable = false;
        AlphaBlendEnable = false;
    }
}
